/* Includes */
#include "stm32f4xx.h"
#include "stm32f4xx_conf.h"

#define ADC1_ADDRESS 0x4001204C;

extern uint8_t CONVERSIONS_COUNT;
extern uint16_t ADC_CONVERSIONS[];


void NVIC_Config();
void GPIO_Config();
void TIM_Config();
void ADC_Config();
void DMA_Config();
void DAC_Config();

int main(void)
{
  DMA_Config();
  GPIO_Config();
  DAC_Config();
  TIM_Config();
  ADC_Config();
  NVIC_Config();

  GPIO_ResetBits(GPIOD, GPIO_Pin_12);
  while (1);
}

void GPIO_Config()
{
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

	//LED
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin		= GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Speed 	= GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode 	= GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType	= GPIO_OType_PP;

	GPIO_Init(GPIOD, &GPIO_InitStructure);

	//Button
	GPIO_InitStructure.GPIO_Pin		= GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Speed 	= GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode 	= GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd	= GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//ADC
	GPIO_InitStructure.GPIO_Pin		= GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Speed 	= GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode 	= GPIO_Mode_AN;
	GPIO_InitStructure.GPIO_PuPd	= GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//DAC
	GPIO_InitStructure.GPIO_Pin		= GPIO_Pin_5;
	GPIO_InitStructure.GPIO_Speed 	= GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode 	= GPIO_Mode_AN;
	GPIO_InitStructure.GPIO_PuPd	= GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void NVIC_Config()
{
	NVIC_InitTypeDef NVIC_Init_Structure;
	EXTI_InitTypeDef EXTI_Init_Structure;
	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

	NVIC_Init_Structure.NVIC_IRQChannel = EXTI0_IRQn;
	NVIC_Init_Structure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_Init_Structure.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init_Structure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_Init_Structure);

	SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource0);
	EXTI_Init_Structure.EXTI_Line = EXTI_Line0;
	EXTI_Init_Structure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_Init_Structure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_Init_Structure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_Init_Structure);

	NVIC_Init_Structure.NVIC_IRQChannel = DMA2_Stream4_IRQn;
	NVIC_Init_Structure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_Init_Structure.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init_Structure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_Init_Structure);
}

void DMA_Config()
{
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

	DMA_InitTypeDef DMA_InitStruct;

	DMA_StructInit(&DMA_InitStruct);
	DMA_InitStruct.DMA_BufferSize = CONVERSIONS_COUNT;
	DMA_InitStruct.DMA_Channel = DMA_Channel_0;
	DMA_InitStruct.DMA_DIR = DMA_DIR_PeripheralToMemory;
	DMA_InitStruct.DMA_Memory0BaseAddr = (uint32_t)ADC_CONVERSIONS;
	DMA_InitStruct.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	DMA_InitStruct.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStruct.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStruct.DMA_PeripheralBaseAddr = (uint32_t)ADC1_ADDRESS;
	DMA_InitStruct.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStruct.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStruct.DMA_Priority = DMA_Priority_Medium;

	DMA_Init(DMA2_Stream4, &DMA_InitStruct);

	DMA_ITConfig(DMA2_Stream4, DMA_IT_TC | DMA_IT_HT, ENABLE);

	DMA_Cmd(DMA2_Stream4, ENABLE);
}

void ADC_Config()
{
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

	ADC_CommonInitTypeDef ADC_CommonInitStruct;

	ADC_CommonInitStruct.ADC_Prescaler = ADC_Prescaler_Div8;
	ADC_CommonInitStruct.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
	ADC_CommonInitStruct.ADC_Mode = ADC_Mode_Independent;
	ADC_CommonInitStruct.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;

	ADC_CommonInit(&ADC_CommonInitStruct);

	ADC_InitTypeDef ADC_Init_Structure;

	ADC_Init_Structure.ADC_ContinuousConvMode = DISABLE;
	ADC_Init_Structure.ADC_DataAlign = ADC_DataAlign_Left;
	ADC_Init_Structure.ADC_Resolution = ADC_Resolution_12b;
	ADC_Init_Structure.ADC_ScanConvMode = DISABLE;
	ADC_Init_Structure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T5_CC1;
	ADC_Init_Structure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_RisingFalling;

	ADC_Init(ADC1, &ADC_Init_Structure);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_3Cycles);

	ADC_DMARequestAfterLastTransferCmd(ADC1, ENABLE);

	ADC_DMACmd(ADC1, ENABLE);

	ADC_Cmd(ADC1, ENABLE);
}

void DAC_Config()
{
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);

	DAC_InitTypeDef DAC_InitStruct;

	DAC_InitStruct.DAC_OutputBuffer = DAC_OutputBuffer_Enable;
	DAC_InitStruct.DAC_Trigger = DAC_Trigger_None;
	DAC_InitStruct.DAC_WaveGeneration = DAC_WaveGeneration_None;

	DAC_Init(DAC_Channel_1, &DAC_InitStruct);

	DAC_Cmd(DAC_Channel_1, ENABLE);

}

void TIM_Config()
{

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);
	TIM_TimeBaseInitTypeDef Tim_InitStructure;
	TIM_OCInitTypeDef Tim_OCInitStructure;

	Tim_InitStructure.TIM_Prescaler = 168-1;
	Tim_InitStructure.TIM_Period = 0xFFFF;
	Tim_InitStructure.TIM_ClockDivision = TIM_CKD_DIV4;
	Tim_InitStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM5, &Tim_InitStructure);

	Tim_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	Tim_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	Tim_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
	Tim_OCInitStructure.TIM_Pulse = 0xFFFF;

	TIM_OC1Init(TIM5, &Tim_OCInitStructure);
}
